@@ -140,6 +140,8 @@ NAMES = {
     'w9a': 'binary/w9a',
     'w9a_test': 'binary/w9a.t',
     'webspam': 'binary/webspam_wc_normalized_trigram.svm.bz2',
+    'YearPredictionMSD': 'regression/YearPredictionMSD.bz2',
+    'YearPredictionMSD_test': 'regression/YearPredictionMSD.t.bz2',
     'yeast': 'multilabel/yeast_train.svm.bz2',
     'yeast_test': 'multilabel/yeast_test.svm.bz2',
 }
@@ -161,6 +163,8 @@ N_FEATURES = {
     'a7a_test': 123,
     'a8a': 123,
     'a8a_test': 123,
+    'a9a': 123,
+    'a9a_test': 123,
     'abalone': 8,
     'abalone_scale': 8,
     'aloi': 128,
@@ -243,6 +247,7 @@ N_FEATURES = {
     'sonar': 60,
     'splice': 60,
     'splice_scale': 60,
+    'splice_test': 60,
     'SUSY': 18,
     'svmguide1': 4,
     'svmguide1_test': 4,
@@ -270,6 +275,8 @@ N_FEATURES = {
     'w9a': 300,
     'w9a_test': 300,
     'webspam': 16_609_143,
+    'YearPredictionMSD': 90,
+    'YearPredictionMSD_test': 90,
     'yeast': 103,
     'yeast_test': 103,
 }
@@ -294,15 +301,15 @@ def get_data_home():
 DATA_HOME = get_data_home()
 
 
-def download_libsvm(dataset, destination, replace=False):
+def download_libsvm(dataset, destination, replace=False, verbose=False):
     """Download a dataset from LIBSVM website."""
     url = ("https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/" +
            NAMES[dataset])
-    path = download(url, destination, replace=replace)
+    path = download(url, destination, replace=replace, verbose=verbose)
     return path
 
 
-def _get_X_y(dataset, multilabel, replace=False):
+def _get_X_y(dataset, multilabel, replace=False, verbose=False):
     """Load a LIBSVM dataset as sparse X and observation y/Y.
     If X and y already exists as npz and npy, they are not redownloaded unless
     replace=True."""
@@ -315,36 +322,46 @@ def _get_X_y(dataset, multilabel, replace=False):
 
     ext = '.npz' if multilabel else '.npy'
     y_path = DATA_HOME / f"{stripped_name}_target{ext}"
-    X_path = DATA_HOME / f"{stripped_name}_data.npz"
-    if replace or not y_path.exists() or not X_path.exists():
+    X_path = DATA_HOME / f"{stripped_name}_data"  # no ext to handle npy or npz
+    if (replace or not y_path.exists()
+        or not ((X_path.parent / (X_path.name + '.npy')).exists() or
+                (X_path.parent / (X_path.name + '.npz')).exists())):
+        # above, do not use .with_suffix bc of datasets like a1a.t, where the
+        # method would replace the .t by .npz
         tmp_path = DATA_HOME / stripped_name
 
         # Download the dataset
         source_path = DATA_HOME / NAMES[dataset]
         if not source_path.parent.exists():
             source_path.parent.mkdir(parents=True)
-        download_libsvm(dataset, source_path, replace=replace)
+        download_libsvm(dataset, source_path, replace=replace, verbose=verbose)
 
         # decompress file only if it is compressed
         if NAMES[dataset].endswith('.bz2'):
             decompressor = BZ2Decompressor()
-            print("Decompressing...")
+            if verbose:
+                print("Decompressing...")
             with open(tmp_path, "wb") as f, open(source_path, "rb") as g:
                 for data in iter(lambda: g.read(100 * 1024), b''):
                     f.write(decompressor.decompress(data))
             source_path.unlink()
 
         n_features_total = N_FEATURES[dataset]
-
-        print("Loading svmlight file...")
+        if verbose:
+            print("Loading svmlight file...")
         with open(tmp_path, 'rb') as f:
             X, y = load_svmlight_file(
                 f, n_features=n_features_total, multilabel=multilabel)
 
         tmp_path.unlink()
-        X = sparse.csc_matrix(X)
-        X.sort_indices()
-        sparse.save_npz(X_path, X)
+        # if X's density is more than 0.5, store it in dense format:
+        if len(X.data) >= 0.5 * X.shape[0] * X.shape[1]:
+            X = X.toarray(order='F')
+            np.save(X_path, X)
+        else:
+            X = sparse.csc_matrix(X)
+            X.sort_indices()
+            sparse.save_npz(X_path, X)
 
         if multilabel:
             indices = np.array([lab for labels in y for lab in labels])
@@ -358,7 +375,11 @@ def _get_X_y(dataset, multilabel, replace=False):
             np.save(y_path, y)
 
     else:
-        X = sparse.load_npz(X_path)
+        try:
+            X = sparse.load_npz(X_path.parent / (X_path.name + '.npz'))
+        except FileNotFoundError:
+            X = np.load(X_path.parent / (X_path.name + '.npy'))
+
         if multilabel:
             y = sparse.load_npz(y_path)
         else:
@@ -367,14 +388,15 @@ def _get_X_y(dataset, multilabel, replace=False):
     return X, y
 
 
-def fetch_libsvm(dataset, replace=False, normalize=False, min_nnz=3):
+def fetch_libsvm(dataset, replace=False, normalize=False, min_nnz=0,
+                 verbose=False):
     """
     Download a dataset from LIBSVM website.
 
     Parameters
     ----------
     dataset : string
-        Dataset name. Must be in .NAMES.keys()
+        Dataset name. Must be in libsvmdata.supported.
 
     replace : bool, default=False
         Whether to force download of dataset if already downloaded.
@@ -384,17 +406,22 @@ def fetch_libsvm(dataset, replace=False, normalize=False, min_nnz=3):
         for a sparse matrix since centering is not performed.
         y is centered and set to unit norm if the dataset is a regression one.
 
-    min_nnz: int, default=3
-        Columns of X with strictly less than min_nnz non-zero entries are
-        discarded.
+    min_nnz : int, default=0
+        When X is sparse, columns of X with strictly less than min_nnz
+        non-zero entries are discarded.
+
+    verbose : bool, default=False
+        Whether or not to print information about dataset loading.
+
 
     Returns
     -------
-    X : scipy.sparse.csc_matrix
-        Design matrix, in column sparse format.
+    X : np.ndarray or scipy.sparse.csc_matrix
+        Design matrix, as 2D array or column sparse format depending on the
+        dataset.
 
-    y : 1D or 2D np.array
-        Design vector or matrix (in multiclass setting)
+    y : 1D or 2D np.ndarray
+        Design vector or matrix (in multiclass setting).
 
 
     References
@@ -403,16 +430,17 @@ def fetch_libsvm(dataset, replace=False, normalize=False, min_nnz=3):
 
     """
     if dataset not in NAMES:
-        raise ValueError("Unsupported dataset %s, check libsvmdata.supported "
-                         "for the list of supported datasets" % dataset)
+        raise ValueError("Unsupported dataset %s. " % dataset +
+                         "Supported datasets are: \n" + ', '.join(NAMES))
     multilabel = NAMES[dataset].split('/')[0] == 'multilabel'
     is_regression = NAMES[dataset].split('/')[0] == 'regression'
 
-    print("Dataset: %s" % dataset)
-    X, y = _get_X_y(dataset, multilabel, replace=replace)
+    if verbose:
+        print("Dataset: %s" % dataset)
+    X, y = _get_X_y(dataset, multilabel, replace=replace, verbose=verbose)
 
-    # preprocessing
-    if min_nnz != 0:
+    # removing columns with to few non zero entries when using sparse X
+    if sparse.issparse(X) and min_nnz != 0:
         X = X[:, np.diff(X.indptr) >= min_nnz]
 
     if normalize:
@@ -422,9 +450,3 @@ def fetch_libsvm(dataset, replace=False, normalize=False, min_nnz=3):
             y /= np.std(y)
 
     return X, y
-
-
-if __name__ == "__main__":
-    for dataset in NAMES:
-        if not dataset.startswith("sector") and not dataset == "webspam":
-            fetch_libsvm(dataset, replace=False)
