@@ -15,10 +15,10 @@ from libsvmdata import fetch_libsvm
 
 def test_binary():
     # download if not present:
-    X, y = fetch_libsvm("news20")
+    X, y = fetch_libsvm("news20.binary")
     np.testing.assert_equal(X.shape[0], y.shape[0])
     # also checks that loading saved files works:
-    X, y = fetch_libsvm("news20")
+    X, y = fetch_libsvm("news20.binary")
 
 
 def test_multilabel():
@@ -39,3 +39,7 @@ def test_multiclass():
     X, y = fetch_libsvm("iris")
     np.testing.assert_equal(X.shape[0], y.shape[0])
     X, y = fetch_libsvm("iris")
+
+
+def test_wrong_dataset():
+    np.testing.assert_raises(ValueError, fetch_libsvm, "unknowndataset")
